#ifndef __COMBSORT_H
#define __COMBSORT_H

#ifndef __SWAP_H
#include "swap.h"
#endif

/*
	Class C of the template allows for the use of different types
	of container classes, whilst T is the type actuall to be
	compared, the compare function is a ptr to function that
	the user creates to compare the types they are using, the
	compare function must return -1 if arg1 is less than arg2,
	0 if they are equal and 1 if arg1 is greater than arg2.
	The only thing is that class C (i.e. Vector< int > must be
	defined and equiped with the [] overloaded operators.

	sf = shrink factor (optimum = 1.3)
*/

template< class Size_t, class C, class T > void combsort( C& array,
Size_t size, signed int (*compare)( T t1, T t2 ), float sf = 1.3 )
{
int	switches;
Size_t	top, j;
Size_t	gap = size;

	do {
		gap = (Size_t)((float)gap / sf );
		switch( gap ) {
		  case 0:
			gap = 1;
			break;
		  case 9:
		  case 10:
			gap = 11;
			break;
		}
		switches = 0;
		top = size - gap;

		for( int i = 0; i < top; i++ ) {
		      j = i + gap;
			if( compare( array[i], array[j] ) > 0 )  {
				swap( array[i], array[j] );
				switches++;
			}
		}
	} while( switches || (gap > 1) );
}

#endif

