Program F1Help;

{ Sample App using a message filter to create context sensitive help
	in a dialog using the F1 function key.  This is a minimal implementation,
	used mainly to demonstrate the creation of a Windows hook.  Enhancements
  are left to the imagination of the programmer. }

{ Written: 7/18/91
			 By: Pat Ritchey CIS ID: [70007,4660]
	Modified 5/25/93 by Scott Steketee [76324.2463]
			to use Windows 3.1 calls to install the hook,
			and to incorporate F1 menu help. }

uses WinTypes,WinProcs,OWindows, ODialogs, Win31;

{$R F1HELP.RES}
{$I F1HELP.INC}

Const
  WM_CtxHelp = WM_User+100;

type
  TMyApplication = object(TApplication)
    procedure InitMainWindow; virtual;
  end;

type
  PMyWindow = ^TMyWindow;
  TMyWindow = object(TWindow)
		MsgInstance: TFarProc;  	{ Holds the ProcInstance of MsgFilter }
		HelpCmd: word;          	{ Keeps track of the currently selected menu item }
		HelpCmdIsHandle: boolean; { True if the selected item is a popup }
		SysMenu: hMenu;           { A copy of the system menu, so we can recognize its selection }
		constructor Init(AParent: PWindowsObject; ATitle: PChar);
		destructor Done; virtual;
		procedure SetupWindow; virtual;
		procedure CMAddCust(var MSg : TMessage); virtual cm_first + cm_AddCust;
		procedure WMMenuSelect (var Msg: TMessage); virtual wm_first + wm_MenuSelect;
		procedure Help(var Msg : TMessage); virtual wm_first + wm_CtxHelp;
		procedure CMQuit(var Msg : TMEssage); virtual cm_first + cm_Quit;
	end;

type
	PHelpDialog = ^THelpDialog;
	THelpDialog = object(TDialog)
		procedure Help(var Msg : TMessage); virtual wm_first + wm_CtxHelp;
	 end;

const
	DisableHelp : boolean = false;

var
		MsgHook: HHook;		{ Handle identifying our hook function }

function MsgFilter(Code : integer; wParam: word; lParam: longInt) : longInt; export;
	{ We are interested only in F1 keypresses while a help dialog is not currently being displayed. }
	begin
	with PMsg (lParam)^ do
		if (Code < 0) or (message <> WM_KEYDOWN) or (wParam <> VK_F1) or DisableHelp
			then MsgFilter := CallNextHookEx (MsgHook, Code, wParam, lParam)
		else
			begin
			{ The user pressed F1 for help in a dialog or menu. }
			{ Send a message to the active window. This will be either the main window }
			{	(for menu help) or the dialog window (for dialog help). Include the hWnd }
			{	parameter to make it easy for the dialog window to tell which of its     }
			{	controls had the focus. }
			{ The value of wParam could be used to distinguish between calls resulting }
			{ from an F1 keypress in a menu (MSGF_MENU) and calls resulting from an F1 }
			{ keypress in a dialog (MSGF_DIALOGBOX). }
			MsgFilter := 1;
			SendMessage(GetActiveWindow, wm_CtxHelp, hWnd,0);
			end;
	end;

{--------------------------------------------------}
{ TMyWindow's method implementations:              }
{--------------------------------------------------}

constructor TMyWindow.Init(AParent: PWindowsObject; ATitle: PChar);
	begin
	TWindow.Init(AParent, ATitle);
	Attr.Menu := LoadMenu(HInstance,'DLGHELP');
	end;

procedure TMyWindow.SetupWindow;
	begin
	inherited SetupWindow;
	{ Install the help filter to detect F1 keypresses in menus and dialogs. }
	DisableHelp := false;
	MsgInstance := MakeProcInstance(@MsgFilter,hInstance);
	MsgHook := SetWindowsHookEx (WH_MSGFILTER, THookProc (MsgInstance), hInstance, GetCurrentTask);
	{ We need a handle to the system menu so we can give help on it. }
	SysMenu := GetSystemMenu (hWindow, false);
	end;

destructor TMyWindow.Done;
	begin
	{ remove the help message filter }
	UnhookWindowsHookEx (MsgHook);
	FreeProcInstance(MsgInstance);
	{ restore the original system menu }
	GetSystemMenu (hWindow, true);
	inherited Done;
	end;

procedure TMyWindow.CMAddCust(Var Msg : TMessage);
	begin
  Application^.ExecDialog(New(PHelpDialog,Init(@Self,'DLGHELP')));
	end;

procedure TMyWindow.WMMenuSelect (var Msg: TMessage);
	{ keep track of what menu item is selected, in case the user asks for help on it. }
	begin
	with Msg do
		begin
		HelpCmd := Msg.wParam;
		HelpCmdIsHandle := (lParamLo and mf_Popup <> 0);
		end;
	end;

function FindCmd (theMenu: hMenu; var theItem: word): boolean;
	{ Walks the given menu, seeing if there are any popups with handle equal to theItem. }
	{ If such a popup is found, returns T, and changes theItem to the position of the popup }
	{ in theMenu. Note that the return value for theItem is 1-based, though the param to }
	{ GetSubMenu is 0-based. }
	var i, id: word;
	begin
	i := 0;
	id := 0;
	while (i <= pred (GetMenuItemCount (theMenu))) and (id <> theItem) do
		begin
		id := GetSubMenu (theMenu, i);
		inc (i);
		end;
	if id = theItem then
		begin
		FindCmd := true;
		theItem := i;
		end
	else FindCmd := false;
	end;	{ FindCmd }

procedure TMyWindow.Help(var Msg: TMessage);
	{ This sample Help method simply uses a message box.  It could
		be modified to call the built in WINHELP facilities }
	{ Msg.lParam and Msg.wParam are unused in this example. }
	const UnknownCmd = 'Unknown menu item selected.';
	var HelpText : packed array [0..255] of char;
	begin  { TMyWindow.Help }
	DisableHelp := true;
	if HelpCmdIsHandle then
		begin   { a popup is currently selected }
		if HelpCmd = SysMenu
			then lStrCpy (HelpText, 'Use the control menu to manipulate the window.')
		else if not FindCmd (Attr.Menu, HelpCmd)
			then lStrCpy (HelpText, UnknownCmd)
		else
			case HelpCmd of
				1: lStrCpy (HelpText, 'Use the File menu to open and close databases, or quit the program.');
				2: lStrCpy (HelpText, 'Use the Customer menu to add customers.');
				else lStrCpy (HelpText, UnknownCmd);
			end;
		end    { popup selected }
	else
		begin  { a command is selected }
		lStrCpy (HelpText, 'Use this command to ');
		case HelpCmd of
			sc_Size:		lStrCat (HelpText, 'change the size of the window.');
			sc_Move:		lStrCat (HelpText, 'move the window.');
			sc_Minimize:lStrCat (HelpText, 'minimize the window.');
			sc_Maximize:lStrCat (HelpText, 'maximize the window.');
			sc_Close:		lStrCat (HelpText, 'close the window.');
			sc_Restore:	lStrCat (HelpText, 'restore the window''s normal size.');
			sc_TaskList:lStrCat (HelpText, 'switch to a different application.');
			cm_Open:    lStrCat (HelpText, 'open a customer data base.');
			cm_Close:   lStrCat (HelpText, 'close the current data base.');
			cm_Quit:		lStrCat (HelpText, 'quit the program.');
			cm_AddCust:	lStrCat (HelpText, 'add a customer to the data base.');
			else				lStrCpy (HelpText, UnknownCmd)
			end;
		end;    { command selected }
	{ do nothing if a separator is selected (HelpCmd = 0) }
	if HelpCmd <> 0
		then MessageBox(hWindow, HelpText, 'Help', mb_Ok);
	DisableHelp := false;
	end; { TMyWindow.Help }

procedure TMyWindow.CMQuit;
	begin
	PostQuitMessage(0);
	end;

procedure THelpDialog.Help(var Msg: TMessage);
	var
		HelpText : Pchar;
	{ This sample Help method simply uses a message box.  It could
		be modified to call the built in WINHELP facilities }
	begin
	{ Msg.wParam contains the window handle of the window with focus }
  { Msg.lPAram is unused in this example. }
	DisableHelp := true;
  case GetDlgCtrlID(MSg.wParam) of
    id_name    : HelpText := 'Enter Customer''s full name';
    id_Address : HelpText := 'Enter Full street address';
    id_City    : HelpText := 'Enter City name';
    id_State   : HelpText := 'Enter State Abbreviation';
    id_Zip     : HelpText := 'Enter 5 or 9 digit Zip code';
		id_OK      : HelpText := 'Press to add customer';
		id_Cancel  : HelpText := 'Press to cancel this entry';
    else
      HelpText := 'Unknown Field';
	end;
  MessageBox(hWindow, HelpText, 'Help', mb_Ok);
  DisableHelp := false;
	end;

{--------------------------------------------------}
{ TMyApplication's method implementations:         }
{--------------------------------------------------}

procedure TMyApplication.InitMainWindow;
	begin
	MainWindow := New(PMyWindow, Init(nil, 'Sample Help Context'));
	end;

{--------------------------------------------------}
{ Main program:                                    }
{--------------------------------------------------}

var
  MyApp : TMyApplication;

begin
MyApp.Init('DLGHELP');
MyApp.Run;
MyApp.Done;
end.
