{
   This is a simple DLL that contains a modeless dialog box indicating the
   status of a task.  I began this project trying to write a DLL that would
   do a file conversion, indicate how it was progressing, and allow the user
   to cancel the process.  I was unable to find a simple example of what I
   was attempting, so I went through the grief of trial and error and
   borrowing from any source that helped.  (If anything looks familiar to
   you, thank-you very much.  Please feel proud that you thought of it
   first, rather than slighted that I don't mention you.)  I learned many
   things along the way and, when I completed my DLL, thought I'd help
   anyone in a similar situation by passing this example on.  This example,
   rather than doing a file conversion, simply counts for 5 seconds.  In my
   file conversion, I display a record count, rather than the second count
   used in this example.  I'm sure, if you need this as an example for a
   file conversion, you will have no trouble adapting it.

   This DLL, though trivial when complete, caused me much grief in the
   making.  I suppose I bit off more than I could chew, learning about
   Windows programming and writing a DLL with a modeless dialog box in it
   all at the same time.  Anyway, if it helps you with a similar project,
   great.  If you have ideas that could make this DLL better, please let me
   know, since I am far from done learning!

   Contributed By: Bradley Plett (71075,2010)
}
LIBRARY statdll;

USES
  strings, wintypes, winprocs, windos, bwcc;

{$R status.res}

VAR
  {variable indicating the user's desire to cancel the task}
  cancelled : boolean;

PROCEDURE upd_disp(Window: HWnd; inint : longint; inperc : REAL);
{
   Procedure to display the percentage as a bar, the percentage as text,
   and a counter
}

  CONST
    rgb_Blue        = $00FF0000;       {blue rectangle interior}
    id_prect = 101;                    {rectangle id}
    id_ptext = 102;                    {% text id}
    id_pcnt = 103;                     {count id}

  VAR
    dc : hdc;                          {device context}
    tmps, tmps2 : ARRAY [0..20] OF char;         {string for output}
    NewBrush,
    OldBrush:  HBrush;                 {brushes used for rectangle interior}
    NewPen  ,
    OldPen  :  HPen;                   {pens used for rectangle outline}
    stat_rect : trect;                 {rectangle for % bar}
    rect_right : integer;              {right coordinate of bar}
    rect_hwnd : hwnd;                  {rectangle window handle}
    tmp_point : tpoint;                {used for coordinate conversion}

  BEGIN   {upd_disp}

    {get device context}
    dc := getdc(window);

    {get rectangle's handle}
    rect_hwnd := GetDlgItem(Window, id_prect);

    {get rectangle's coordinates}
    GetwindowRect(rect_hwnd, stat_rect);

    {convert coordinates to be relative to the dialog box}
    WITH stat_rect DO
      BEGIN
        tmp_point.x := left;
        tmp_point.y := top;
        ScreenToClient(window, tmp_point);
        left := tmp_point.x + 1;
        top := tmp_point.y + 1;
        tmp_point.x := right;
        tmp_point.y := bottom;
        ScreenToClient(window, tmp_point);
        right := tmp_point.x - 1;
        bottom := tmp_point.y - 1;
      END;   {with}

    {save old brush and select a new one for painting the bar}
    NewBrush := CreateSolidBrush(rgb_blue);
    IF (NewBrush = 0) THEN   { Use the new brush if it was made }
      OldBrush := 0
    ELSE
      OldBrush := SelectObject(DC, NewBrush);

    {save old pen and select a new one for the bar's outline}
    NewPen := CreatePen(ps_Solid, 1, rgb_blue);
    IF (NewPen = 0) THEN
      OldPen := 0
    ELSE
      OldPen := SelectObject(DC, NewPen);

    {calculate the bar's right coordinate}
    rect_right := round(inperc / 100.0 *
                        (stat_rect.right - stat_rect.left) +
                        stat_rect.left);

    {draw the bar}
    rectangle(dc, stat_rect.left, stat_rect.top,
                  rect_right, stat_rect.bottom);

    {restore the old brush and pen}
    IF (OldBrush <> 0) THEN   { Restore the original brush now! }
      BEGIN
        SelectObject(DC, OldBrush);
        DeleteObject(NewBrush)
      END;   {if}
    IF (OldPen <> 0) THEN
      BEGIN
        SelectObject(DC, OldPen);
        DeleteObject(NewPen)
      END;   {if}

    {display the % complete in text form}
    str(inperc:1:0, tmps);
    strcat(tmps, '%');
    SetDlgItemText(Window, id_ptext, tmps);

    {display the counter}
    str(inint:1, tmps2);
    strcopy(tmps, 'Count: ');
    strcat(tmps, tmps2);
    SetDlgItemText(Window, id_pcnt, tmps);

    {release the device context}
    releasedc(window, dc);

  END;   {upd_disp}

FUNCTION MyDlgProc(Dlg : hWnd; msg : word; wParam : word; lParam : longint) : longint; EXPORT;
{
   Function to handle the dialog box messages
}
  BEGIN   {MyDlgProc}
    MyDlgProc := 0;
    CASE msg OF
        wm_InitDialog :
          BEGIN
            {get rid of the "Close" menu option}
            DeleteMenu(GetSystemMenu(dlg, false), sc_close, mf_bycommand);
            MyDlgProc := -1;
          END;   {wm_InitDialog}
        wm_Command : BEGIN
            {if the user selected "Cancel", set the global variable}
            IF (wParam = id_Cancel) THEN
              cancelled := true;
          END;   {wm_Command}
      END;   {case}
  END;   {MyDlgProc}

FUNCTION Task_Stat : integer; EXPORT;
{
   Function that, in this example, contains the main loop.  It counts for
   five seconds or until the user selects "Cancel".  It returns "0" if
   cancelled (false) and "-1" (true) if the task completes successfully.
}

  CONST
    myhwnd : hwnd = 0;
    stop_at = 5;                       {seconds to run}

  VAR
    all_done,                          {indicates completion}
    good_finish : boolean;             {indicates successful completion}
    curcnt,                            {current millisecond count}
    startcnt : longint;                {count at beginning of loop}
    curperc : REAL;                    {current percentage}
    hmain : hwnd;                      {dialog box handle}
    myproc : tFarProc;                 {message handling procedure}
    message : tmsg;                    {messages to process}

  BEGIN   {Task_Stat}

    {initialize variables}
    Task_Stat := 0;
    good_finish := false;
    cancelled := false;

    {set up message handling procedure}
    MyProc := MakeProcInstance(@MyDlgProc, hInstance);

    {create the dialog box}
    hmain := CreateDialog(hInstance, 'Status', myhwnd, MyProc);
    IF (hmain = 0) THEN
      messagebox(0, 'Could not create a window', 'Error', mb_iconstop + mb_ok)
    ELSE
      BEGIN

        {display and enable the dialog box}
        ShowWindow(hmain, sw_show);
        EnableWindow(myhwnd, false);

        {initialize and begin loop}
        startcnt := GetTickCount;
        all_done := false;
        WHILE (NOT all_done) DO
          BEGIN

            {check if there's a message waiting}
            IF PeekMessage(Message, myhwnd, 0, 0, pm_remove) THEN
              BEGIN
                {handle messages for the dialog box and pass on others}
                IF (hmain = 0) OR (NOT isdialogmessage(hmain, message)) THEN
                  BEGIN
                    TranslateMessage(Message);
                    DispatchMessage(Message);
                  END;   {if}
              END
            ELSE
              BEGIN
                {***
                for file conversion, place the record conversion part here
                ***}
                {increment the count}
                curcnt := GetTickCount - startcnt;
                curperc := 100.0 * (curcnt / (stop_at * 1000.0));
                IF (curcnt >= (stop_at * 1000)) THEN
                  {the task has been completed successfully}
                  good_finish := true
                ELSE
                  {the task is still in progress - update the display}
                  upd_disp(hmain, round(curcnt / 10.0), curperc);
              END;   {else}
            {conditions for ending: cancelled or finished}
            all_done := cancelled OR good_finish;
          END;   {while}

        {get rid of the dialog box}
        EnableWindow(myhwnd, true);
        DestroyWindow(hmain);

      END;   {if}

    {get rid of the message handling procedure's handle}
    FreeProcInstance(MyProc);

    {if successfully completed, return "-1" (true)}
    IF good_finish THEN
      Task_Stat := -1;

  END;   {Task_Stat}

EXPORTS
  Task_Stat index 1;

BEGIN   {statdll}
END.   {statdll}